/**************************************************************
 *
 *
 * $Header:$
 *
 * $Log:$
 *
 * MODULE DESCRIPTION
 *
 *   Sample from top layer of network, given states in layer below
 *   
 *
 *
 * $Author:$
 *
 *************************************************************/
#include <stdlib.h>
#include <math.h>
#include <mex.h>
#include "twogauss.h"
#include "threegauss.h"
#include "topsample.h"

static  double  pi2 = 2.0 * M_PI;
static  double  tiny = 10 / HUGE;

#define MAX(x,y)	(x > y ? x : y)
#define MIN(x,y)	(x < y ? x : y)
#define REAL		(0)

/************************************************************
 *
 * sample from top layer given middle layer
 *
 * x1 - State of lop layer
 * x2  - x2 - bias at lower layer
 * W12 - Weight matrix between layers
 * b1, b2 - Bias at top layer and lower layer
 * v1, v2  - Variances at top lower layer
 *
 * Returns:  x1New - The new state x1
 *
 *************************************************************/
Matrix * topsample(
int		nrhs,
Matrix		*prhs[],
double		*newX1r
)
{
  Matrix	*x1, *x2, *W12, *b1, *b2, *v1, *v2, *nl;
  Matrix	*x1r = NULL, *ret = NULL, *y;
  int		i = 0, j;
  int		ns, totSamp;
  double	mua, va, md, vd;
  int		n1, n2, NL;

  /* Pointers to matrix data */
  double	*pM, *pb1, *pb2, *px2, *px1, *px1r, *pv1, *pv2, *pW12;
  double	*pRet, *py;


  x1 = prhs[i++];
  x2 = prhs[i++];
  W12 = prhs[i++];
  b1 = prhs[i++];
  b2 = prhs[i++];
  v1 = prhs[i++];
  v2 = prhs[i++];
  nl = prhs[i++];

  n1 = mxGetM(x1);
  n2 = mxGetM(x2);

  if (mxGetM(W12) != n2 && mxGetN(W12) != n1)
  {
    mexErrMsgTxt("topsample: W12 does not match dimensions of x1 and x2");
  }

  x1r = mxCreateFull(n1, 1, REAL);
  y = mxCreateFull(n2, 1, REAL);

  pb1 = mxGetPr(b1);
  pb2 = mxGetPr(b2);
  px1 = mxGetPr(x1);
  px2 = mxGetPr(x2);
  pv1 = mxGetPr(v1);
  pv2 = mxGetPr(v2);
  px1r = mxGetPr(x1r);
  pW12 = mxGetPr(W12);
  NL = (int) *mxGetPr(nl);
  py = mxGetPr(y);

  ret = mxCreateFull(n1, 1, REAL);

  pRet = mxGetPr(ret);

  /* rectified top activities */
  if (NL==1)
    for (j = 0 ; j < n1 ; ++j)
      {
	px1r[j] = MAX(px1[j], 0.0);
      }
  else if (NL==3)
    for (j = 0 ; j < n1 ; ++j)
      {
	px1r[j] = MAX(px1[j], 0.0);
	px1r[j] = MIN(px1r[j], 1.0);
      }

  for ( j = 0 ; j < n1 ; ++j)
  {
    /* top down mean and var */
    mua = pb1[j];		/* mean is bias */
    va = *pv1;      		


    /* Combine top down with bottom up */
    for (i = 0 ; i < n2 ; ++i)
    {
      py[i] = px2[i] - dotProd(W12, i, x1r, 0) - pb2[i] + pW12[j*n2 + i] * px1r[j];
    }

    vd = 1 / (1/va + transDotProd(W12, j, W12, j) / *pv2);

    md = vd * (mua/va + transDotProd(y, 0, W12, j) / *pv2);
  
#ifdef DEBUG
    printf("%d %f %f0\n", j, md, vd);
#endif

    if (NL == 1)
    {
      px1[j] =twogauss(mua,va,md,vd);
      px1r[j] = MAX(px1[j], 0.0);
    }
    else if (NL==3)
      {
	px1[j] =threegauss(mua,va,md,vd);
	px1r[j] = MAX(px1[j], 0.0);
	px1r[j] = MIN(px1r[j], 1.0);
      }
  
    pRet[j] = px1[j];
  }
  return (ret);
}


void mexFunction(
int		nlhs,
Matrix		*plhs[],
int		nrhs,
Matrix		*prhs[]
)
{
  double	*retx1;
  int		i = 0;
  double	*retX1;
  static int	expectRhs = 8;
  static int	expectLhs = 1;
  char		buffer[100];

  /* Check for proper number of arguments */
  
  if (nrhs != expectRhs) 
  {
    sprintf(buffer, "topsample requires %d input arguments. Got %d",
	    expectRhs, nrhs);
    mexErrMsgTxt(buffer);
  } else if (nlhs > 1) 
  {
    sprintf(buffer, "topsample requires %d outputs arguments. Got %d",
	    expectLhs, nlhs);
    mexErrMsgTxt(buffer);
  }

  plhs[0] = topsample(nrhs, prhs, retX1);
}
