NL=1;         % nonlinearity of the unit(s) at the top layer: 1=rectified, 3=squashed 
              % In the simulation presented in the Phil Trans Roy Soc paper NL=3,
	      % However, the mex file for threegauss hasn't been implemented yet, 
	      % so to get a fast simulation you need to set NL=1.

% number of units

P=18;         % number of pixels in each retina
n1=1;         % number of units at top layer 
n2=24;        % numb of hidden units                
n3=2*P;       % numb of visible units

reps=16;      % Gibbs samples

eta=0.1;      % learning rate
lambda=0.001  % weight decay parameter
lambdav=0.001 % weight decay parameter for variances

eta2=eta;     % learning rate at top layer
eta3=eta;     % learning rate at bottom layer

numparam=n2*n1+n3*n2+n1+n2+n3+3; % total numb of params

savestep=200;   % save weights every this many steps
graphstep=400;  % plot progress graphics every this many steps (multiple of savestep)

init=1;       % flag to specify whether to re-initialize params or not
if init,      % initialize params and activities

  rgbn_init;
  
  %%% generate stereo data
  
  N=1000;                       % number of data points
  [Data, Di, Dd]=genstereo(N,P,0,0,0.2);   % generate this data
  Data=Data*5; Di=Di*5;
  cycles=10000;                 % number of data presentations
  params=zeros(fix(cycles/savestep),numparam); % record of all params during learning
  activ=zeros(N,n1+n2);
else
  initcyc=cycles;              
  cycles=5000;
  params=[params; zeros(fix(cycles/savestep),numparam)];  % append to old params
end;

%%%

tic;

for cyc=initcyc+1:initcyc+cycles,     % cycle over the data cycles times
  
  %% CLAMP ONE DATA VECTOR AND LEARN
  
  rgbn_core;

  if rem(cyc,20)==0,
    fprintf('.');
  end;
  
  if rem(cyc,savestep)==0,  % every savestep-th cycle print and collect params
    fprintf(' %g ',cyc);
    params(fix(cyc/savestep),:)=[W12(:)' W23(:)' b1(:)' b2(:)' b3(:)' v1 v2 v3];
    if rem(cyc,graphstep)==0,   % every graphstep-th cycle show plot of what's going on
      showstereo;
    end;

  end;
end;
toc;

